/*
 * File: hash_table.h
 * ------------------
 * Hash table.
 *
 * By: Marcus 2018
 */

#ifndef __HASH_TABLE_H__
#define __HASH_TABLE_H__

#define HT_REF_COUNT

#define HT_DEF_CAPACITY 1
#define HT_GROW_AT 0.75
#define HT_SHRINK_AT 0.25


/*
 * Struct: HashEntry
 * -----------------
 */
typedef struct _HashEntry HashEntry;
struct _HashEntry{
    void *data;
    char *key;
    HashEntry *next;
};

/*
 * Struct: HashTable
 * -----------------
 */
typedef struct {
    int capacity;
    int minCapacity;
    int entries;
    HashEntry **list;
} HashTable;

/*
 * Function: HT_Create
 * -------------------
 */
HashTable *HT_Create(unsigned int capacity);

/*
 * Function: HT_Clear
 * ------------------
 */
void HT_Clear(HashTable *ht, void (*delFunc)(void *));

/*
 * Function: HT_Free
 * -----------------
 */
void HT_Free(HashTable *ht, void (*delFunc)(void *));

/*
 * Function: ApplyDataFunction
 * ---------------------------
 */
void HT_ApplyDataFunction(HashTable *ht, void (*func)(void *));

/*
 * Function: HT_ApplyKeyFunction
 * -----------------------------
 */
void HT_ApplyKeyFunction(HashTable *ht, void (*func)(char *, void *));

/*
 * Function: HT_Hash
 * -----------------
 */
unsigned int HT_Hash(HashTable *ht, const char *key);

/*
 * Function: HT_Add
 * ----------------
 */
int HT_Add(HashTable *ht, const char *key, void *data);

/*
 * Function: HT_UnsafeAdd
 * ----------------------
 * No checking if key already exists.
 */
void HT_UnsafeAdd(HashTable *ht, const char *key, void *data);

/*
 * Function: HT_AddEntry
 * ---------------------
 * Used when resizing.
 */
void HT_AddEntry(HashTable *ht, HashEntry *e);

/*
 * Function: HT_Delete
 * -------------------
 */
int HT_Delete(HashTable *ht, const char *key, void (*delFunc)(void *));

/*
 * Function: HT_Function
 * ---------------------
 */
int HT_Function(HashTable *ht, const char *key, void (*func)(void *));

/*
 * Function: HT_Get
 * ----------------
 */
void *HT_Get(HashTable *ht, const char *key);

/*
 * Function: HT_GetKey
 * -------------------
 */
int HT_GetKey(HashTable *ht, const void *data, char *key);

/*
 * Function: HT_Exists
 * -------------------
 */
int HT_Exists(HashTable *ht, const char *key);

/*
 * Function: HT_Resize
 * -----------------------
 */
void HT_Resize(HashTable *ht, unsigned int capacity);


int HT_TableCount();

#endif
